#include "RAEdit_VC.h"

//Registers
TCHAR const szRegs[] = TEXT("AH AL AX BH BL BP BX CH CL CR0 CR2 CR3 CS CX DH DI DL \
DR0 DR1 DR2 DR3 DR6 DR7 DS DX EAX EBP EBX ECX EDI EDX \
ES ESI ESP FS GS MM0 MM1 MM2 MM3 MM4 MM5 MM6 MM7 \
SI SP SS ST TR3 TR4 TR5 TR6 TR7 XMM0 XMM1 XMM2 XMM3");

// Words to hilite
TCHAR const szInst[] = TEXT("AAA AAD AAM AAS ADC ADD AND CALL CBW CLC CLD CLI CMC \
CMP CMPS CMPSB CMPSW CWD DAS DEC DIV DAA ESC HLT IDIV IMUL IN INC INT INTO IRET \
JA JAE JB JBE JC JCXZ JE JG JGE JL JLE JMP JNA JNAE JNB JNBE JNC JNE JNG JNGE JNL \
JNLE JNO JNP JNS JNZ JO JP JPE JPO JS JZ LAHF LDS LEA LES LODS LODSB LODSW \
LOOP LOOPE LOOPEW LOOPNE LOOPNEW LOOPNZ LOOPNZW LOOPW LOOPZ LOOPZW \
MOV MOVS MOVSB MOVSW MUL NEG NOP NOT OR OUT POP POPF PUSH PUSHF RCL RCR \
RET RETF RETN ROL ROR SAHF SAL SAR SBB SCAS SCASB SCASW SHL SHR STC STD STI \
STOS STOSB STOSW SUB TEST WAIT XCHG XLAT XLATB XOR");

TCHAR const szMasm[] = TEXT(".BREAK .CONTINUE .ELSE .ELSEIF .ENDIF .ENDW .IF \
.REPEAT .UNTIL .UNTILCXZ .WHILE INVOKE");


TCHAR szMainWindowClass[] = TEXT("MainWindowClassName"); 

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE, PSTR, INT iCmdShow)
{
    HINSTANCE           hRAEditDLL=NULL;           // Handle to DLL
    HWND                hWnd;
    MSG                 msg;
    WNDCLASS            wndClass; 

    InitCommonControls(); 
    hRAEditDLL = LoadLibrary(TEXT("RAEdit.dll"));
    if ( hRAEditDLL ) { 
        wndClass.style          = CS_HREDRAW | CS_VREDRAW;
        wndClass.lpfnWndProc    = WndProc;
        wndClass.cbClsExtra     = 0;
        wndClass.cbWndExtra     = 0;
        wndClass.hInstance      = hInstance;
        wndClass.hIcon          = LoadIcon(NULL, IDI_APPLICATION);
        wndClass.hCursor        = LoadCursor(NULL, IDC_ARROW);
        wndClass.hbrBackground  = (HBRUSH)GetStockObject(WHITE_BRUSH);
        wndClass.lpszMenuName   = MAKEINTRESOURCE(IDR_MAIN_MENU);
        wndClass.lpszClassName  = szMainWindowClass;

        RegisterClass(&wndClass);

        hWnd = CreateWindow(
            szMainWindowClass,        // window class name
            TEXT("RadASM Edit Control Powered Editor"),  // window caption
            WS_OVERLAPPEDWINDOW,      // window style
            CW_USEDEFAULT,            // initial x position
            CW_USEDEFAULT,            // initial y position
            CW_USEDEFAULT,            // initial x size
            CW_USEDEFAULT,            // initial y size
            NULL,                     // parent window handle
            (HMENU)NULL,              // window menu handle
            hInstance,                // program instance handle
            NULL);                    // creation parameters

        ShowWindow(hWnd, iCmdShow);
        UpdateWindow(hWnd);

        while(GetMessage(&msg, NULL, 0, 0))
        {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }

        FreeLibrary(hRAEditDLL); 

        return static_cast<int>(msg.wParam) ;
    } else { 
        MessageBox( NULL, 
                    TEXT("Could not find RAEdit.dll"), 
                    TEXT("RAEdit_VC"), 
                    MB_OK | MB_ICONEXCLAMATION); 
        return 0; 
    }
}  // WinMain


LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    //HDC          hdc;
    //PAINTSTRUCT  ps;
    RECT rect; 
    static HWND hRaEditWnd = NULL; 
    static TCHAR szFileName[MAX_PATH] = TEXT("NoName.asm"); 
    static CREATESTRUCT *lpCreateStruct; 
    static RACOLOR raColor; 
    static RAFONT raf;

    switch(message)
    {
    case WM_PAINT:
        //hdc = BeginPaint(hWnd, &ps);
        //OnPaint(hdc);
        //EndPaint(hWnd, &ps);
        break; 
    case WM_COMMAND: // Controling the Buttons
        switch (LOWORD(wParam)) // what we pressed on?
        { 	 			    
        case ID_HELP_ABOUT: 
            DialogBox(  lpCreateStruct->hInstance, 
                    MAKEINTRESOURCE(IDD_DIALOG_ABOUT), //TEXT("DIALOG_ABOUT"),  
                    hWnd, 
                    (DLGPROC)AboutDialogProc) ; 
            GetLastError(); 
            break;
        case ID_FILE_OPEN:
            {
                OPENFILENAME ofn; 
                RtlZeroMemory(&ofn, sizeof(OPENFILENAME)); 
                ofn.lStructSize = sizeof(OPENFILENAME); 
                ofn.hwndOwner = hWnd; 
                ofn.lpstrFile = szFileName; 
                ofn.hInstance = lpCreateStruct->hInstance; 
                ofn.nMaxFile = MAX_PATH; 
                ofn.Flags = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_PATHMUSTEXIST; 
                ofn.lpstrDefExt = TEXT("asm");
                ofn.lpstrFilter = TEXT("Asm files (*.asm, *.inc)\0*.asm;*.inc\0All Files (*.*)\0*.*\0");
                if (GetOpenFileName(&ofn)) { 
                    HANDLE hFile = CreateFile(  szFileName, 
                                                GENERIC_READ, 
                                                FILE_SHARE_READ, 
                                                NULL, 
                                                OPEN_EXISTING, 
                                                FILE_ATTRIBUTE_NORMAL, 
                                                NULL ); 
                    if (hFile != INVALID_HANDLE_VALUE) {  
                        LoadFileToRadAsmEd(hRaEditWnd, hFile); 
                        CloseHandle(hFile);
                    }
                }
                break;
            }
        case ID_FILE_SAVE:
            {
                SaveFileFromRadAsmEd(hRaEditWnd, szFileName);
                break;
            }
        case ID_FILE_SAVEAS:
            {
                SaveFileAsFromRadAsmEd(hRaEditWnd, szFileName);
                break;
            }
        case ID_FILE_EXIT: 
            SendMessage(hWnd,WM_CLOSE,0,0); 
            break; 
        case ID_EDIT_UNDO: 
            SendMessage(hRaEditWnd, EM_UNDO, 0, 0); 
            break; 
        case ID_EDIT_REDO: 
            SendMessage(hRaEditWnd, EM_REDO, 0, 0); 
            break; 
        case ID_EDIT_CUT: 
            SendMessage(hRaEditWnd, WM_CUT, 0, 0); 
            break;
        case ID_EDIT_COPY: 
            SendMessage(hRaEditWnd, WM_COPY, 0, 0); 
            break; 
        case ID_EDIT_PASTE: 
            SendMessage(hRaEditWnd, WM_PASTE, 0, 0); 
            break; 
        case ID_EDIT_DELETE: 
            SendMessage(hRaEditWnd, WM_CLEAR, 0, 0); 
            break; 
        case ID_EDIT_SELECTALL: 
            CHARRANGE       ChRg; 
            ZeroMemory(&ChRg, sizeof(CHARRANGE)); 
            ChRg.cpMin = 0; 
            ChRg.cpMax = -1; 
            SendMessage(hRaEditWnd, EM_EXSETSEL, 0, (LPARAM)&ChRg); 
            break;
        }
        break;
    case WM_SIZE: 
        GetClientRect(hWnd, &rect); 
        MoveWindow(hRaEditWnd, 0, 0, rect.right, rect.bottom, TRUE); 
        break; 
    case WM_CLOSE: 
        if(SendMessage(hRaEditWnd, EM_GETMODIFY, 0, 0)==TRUE)
        { 
            switch( MessageBox( hWnd,
                        TEXT("There Has Been Changes, Do You Want To Save Them?"),
                        TEXT("Save Changes"),
                        MB_YESNOCANCEL|MB_ICONQUESTION)) 
            {
            case IDYES:
                SaveFileFromRadAsmEd(hRaEditWnd, szFileName);
                // Fall through 
            case IDNO: 
                break; 
            case IDCANCEL: 
                return 0; 
            }
        } 
        SendMessage(hWnd, WM_DESTROY, wParam, lParam); 
        break; 
    case WM_DESTROY:
        PostQuitMessage(0);
        DeleteObject(raf.hFont); 
        DeleteObject(raf.hIFont); 
        DeleteObject(raf.hLnrFont); 
        break; 
    case WM_CREATE: 
        lpCreateStruct = (CREATESTRUCT *)lParam; 
        hRaEditWnd = CreateWindowEx(WS_EX_CLIENTEDGE, szRaEditClassName, NULL, 
            WS_CHILD|WS_VISIBLE, 
            0, 0, CW_USEDEFAULT, CW_USEDEFAULT, hWnd, lpCreateStruct->hMenu,
            lpCreateStruct->hInstance, NULL);         
        if ( !hRaEditWnd ) 
        { 
            MessageBox(NULL, TEXT("Can not Create RAEdit Window"), TEXT("Error"), 
                MB_OK | MB_ICONEXCLAMATION); 
        }
        SendMessage (hRaEditWnd, REM_SETHILITEWORDS, RGB(0,0,0),   0 );
        SendMessage (hRaEditWnd, REM_SETHILITEWORDS, RGB(255,0,0), (LPARAM)szRegs);
        SendMessage (hRaEditWnd, REM_SETHILITEWORDS, RGB(0,100,100), (LPARAM)szInst );
        SendMessage (hRaEditWnd, REM_SETHILITEWORDS, RGB(0,0,255), (LPARAM)szMasm );

        LOGFONT lf; 
        RtlZeroMemory(&lf, sizeof(LOGFONT)); 
        lf.lfHeight = -12; 
        _tcscpy( lf.lfFaceName, TEXT("Courier New")); 
        raf.hFont = CreateFontIndirect(&lf); 
        lf.lfItalic = TRUE; 
        raf.hIFont = CreateFontIndirect(&lf); 
        lf.lfHeight = -8; 
        _tcscpy(lf.lfFaceName, TEXT("Terminal")); 
        lf.lfItalic = FALSE; 
        raf.hLnrFont = CreateFontIndirect(&lf); 
        SendMessage(hRaEditWnd, REM_SETFONT, 0, (LPARAM)&raf); 

        // Set Colors 
        RtlZeroMemory(&raColor, sizeof(RACOLOR)); 
        SendMessage(hRaEditWnd, REM_GETCOLOR, 0, (LPARAM)&raColor);  // Get all default colors 
        raColor.oprcol = RGB(0, 0, 255);    // Change some colors  
        SendMessage(hRaEditWnd, REM_SETCOLOR, 0, (LPARAM)&raColor); 

        break; 
    default:
        return DefWindowProc(hWnd, message, wParam, lParam);
    } 
    return 0;
} // WndProc

void SaveFileFromRadAsmEd(HWND hRadEditHWnd, TCHAR *szFileName)
{
    // content of edit control has been modified?
    if(TRUE == SendMessage(hRadEditHWnd, EM_GETMODIFY, 0, 0))
    {
        SaveFile(hRadEditHWnd, szFileName);
    }
    else MessageBox(hRadEditHWnd,"Nothing To Save, No Changes!","Opps",MB_OK);
}

void SaveFileAsFromRadAsmEd(HWND hRadAsmEdWnd, TCHAR *lpszFileName)
{
//    HWND hRadAsmEdWnd = GetDlgItem(hWnd, IDC_RAHEXEDIT);
    TCHAR szFileNameSave[MAX_PATH] = TEXT("");
    OPENFILENAME ofn;

    ZeroMemory(&ofn, sizeof(OPENFILENAME)); // Init the struct to Zero
    ofn.lStructSize = sizeof(OPENFILENAME); // SEE NOTE BELOW
    ofn.hwndOwner =  GetWindow(hRadAsmEdWnd, GW_OWNER); 
    ofn.lpstrFilter = TEXT("Asm files (*.asm, *.inc)\0*.asm;*.inc\0All Files (*.*)\0*.*\0");
    ofn.lpstrFile = szFileNameSave;
    ofn.nMaxFile = MAX_PATH;
    ofn.Flags = OFN_OVERWRITEPROMPT | OFN_PATHMUSTEXIST | OFN_HIDEREADONLY | OFN_FILEMUSTEXIST;
    ofn.lpstrDefExt = TEXT("asm");

    // content of edit control has been modified?
    if(TRUE == SendMessage(hRadAsmEdWnd,EM_GETMODIFY,0,0))
    {
        if(GetSaveFileName(&ofn)!=NULL)
        {
            SaveFile(hRadAsmEdWnd,szFileNameSave); 
            _tcscpy(lpszFileName, szFileNameSave);
        }
        else return;
    }
    else MessageBox(hRadAsmEdWnd, TEXT("Nothing To Save, No Changes!"), TEXT("Opps"), MB_OK);
}


void SaveFile(HWND hRadWin,TCHAR *lpszFileName)
{
    HANDLE hFile;

    hFile = CreateFile(lpszFileName,
        GENERIC_WRITE,
        FILE_SHARE_READ,
        NULL,
        OPEN_ALWAYS,
        FILE_ATTRIBUTE_NORMAL,
        NULL );

    if(hFile == INVALID_HANDLE_VALUE) return;

    // Init the struct to Zero
    EDITSTREAM editstream;    
    ZeroMemory(&editstream, sizeof(EDITSTREAM));
    editstream.dwCookie = (DWORD_PTR)hFile;
    editstream.pfnCallback = StreamOutProc;
    SendMessage(hRadWin, EM_STREAMOUT, (WPARAM)SF_TEXT, (LPARAM)&editstream);

    CloseHandle(hFile);
    SendMessage(hRadWin, EM_SETMODIFY, FALSE, 0);
}

DWORD CALLBACK StreamOutProc(DWORD dwCookie, LPBYTE lpbBuff, LONG cb, LONG FAR *pcb)
{
    if (!WriteFile((HANDLE)dwCookie, (LPVOID)lpbBuff, cb, (LPDWORD)pcb, NULL))
        return((DWORD)-1);
    return(0);
}

BOOL CALLBACK AboutDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam) 
{ 
    switch(uMsg) {
    case WM_INITDIALOG: 
    	break;
    case WM_CLOSE: 
        EndDialog ( hwndDlg, 0 );
    	break;
    case WM_COMMAND:
        switch (LOWORD (wParam))
        {
        case IDOK:
            // ... 
            // Fall through. 
        case IDCANCEL: 
            EndDialog (hwndDlg, wParam) ;
        }
        break ;
    default: 
        return FALSE; 
        break;
    }
    return TRUE; 
}

void LoadFileToRadAsmEd(HWND hRadAsmEdWnd, HANDLE hFileArg) 
{ 
    EDITSTREAM      editstream;
    ZeroMemory(&editstream, sizeof(EDITSTREAM));
    editstream.dwCookie = (DWORD_PTR)hFileArg;
    editstream.pfnCallback = StreamInProc; 
    SendMessage (hRadAsmEdWnd, WM_SETTEXT, 0, 0 );
    SendMessage(hRadAsmEdWnd, EM_STREAMIN, (WPARAM)SF_TEXT, (LPARAM)&editstream);

    SendMessage(hRadAsmEdWnd, EM_SETMODIFY, FALSE, 0);

    CHARRANGE       chrg;
    ZeroMemory(&chrg, sizeof(CHARRANGE));
    chrg.cpMin=0;
    chrg.cpMax=0;
    SendMessage(hRadAsmEdWnd,EM_EXSETSEL,0,(LPARAM)&chrg);
}

DWORD CALLBACK StreamInProc(DWORD_PTR dwCookie, LPBYTE lpbBuff, LONG cb, LONG FAR *pcb)
{
    if (!ReadFile((HANDLE)dwCookie, (LPVOID)lpbBuff, cb, (LPDWORD)pcb, NULL))
        return((DWORD)-1);
    return(0);
}

